@@ -1,26 +1,26 @@
 #ifdef MALLOCDEBUG
-/*****************************************************************************
+/***************************************************************************
  *                                  _   _ ____  _     
  *  Project                     ___| | | |  _ \| |    
  *                             / __| | | | |_) | |    
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
- *
- * In order to be useful for every potential user, curl and libcurl are
- * dual-licensed under the MPL and the MIT/X-derivate licenses.
+ * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
+ * This software is licensed as described in the file COPYING, which
+ * you should have received as part of this distribution. The terms
+ * are also available at http://curl.haxx.se/docs/copyright.html.
+ * 
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
- * furnished to do so, under the terms of the MPL or the MIT/X-derivate
- * licenses. You may pick one of these licenses.
+ * furnished to do so, under the terms of the COPYING file.
  *
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: memdebug.c,v 1.19 2001/11/28 23:19:17 bagder Exp $
- *****************************************************************************/
+ * $Id: memdebug.c,v 1.26 2002/09/03 11:53:00 bagder Exp $
+ ***************************************************************************/
 
 #include "setup.h"
 
@@ -47,6 +47,13 @@
 
 /* DONT include memdebug.h here! */
 
+struct memdebug {
+  int size;
+  double mem[1];
+  /* I'm hoping this is the thing with the strictest alignment
+   * requirements.  That also means we waste some space :-( */
+};
+
 /*
  * Note that these debug functions are very simple and they are meant to
  * remain so. For advanced analysis, record a log file and write perl scripts
@@ -67,16 +74,25 @@ void curl_memdebug(const char *logname)
 }
 
 
-void *curl_domalloc(size_t size, int line, const char *source)
+void *curl_domalloc(size_t wantedsize, int line, const char *source)
 {
-  void *mem=(malloc)(size);
-  if(mem)
+  struct memdebug *mem;
+  size_t size;
+
+  /* alloc at least 64 bytes */
+  size = sizeof(struct memdebug)+wantedsize;
+
+  mem=(struct memdebug *)(malloc)(size);
+  if(mem) {
     /* fill memory with junk */
-    memset(mem, 0xA5, size);
-  if(logfile)
+    memset(mem->mem, 0xA5, wantedsize);
+    mem->size = wantedsize;
+  }
+
+  if(logfile && source)
     fprintf(logfile, "MEM %s:%d malloc(%d) = %p\n",
-            source, line, size, mem);
-  return mem;
+            source, line, wantedsize, mem->mem);
+  return mem->mem;
 }
 
 char *curl_dostrdup(const char *str, int line, const char *source)
@@ -90,32 +106,56 @@ char *curl_dostrdup(const char *str, int line, const char *source)
     exit(2);
   }
 
-  mem=(strdup)(str);
   len=strlen(str)+1;
+
+  mem=curl_domalloc(len, 0, NULL); /* NULL prevents logging */
+  memcpy(mem, str, len);
+
   if(logfile)
     fprintf(logfile, "MEM %s:%d strdup(%p) (%d) = %p\n",
             source, line, str, len, mem);
+
   return mem;
 }
 
-void *curl_dorealloc(void *ptr, size_t size, int line, const char *source)
+void *curl_dorealloc(void *ptr, size_t wantedsize,
+                     int line, const char *source)
 {
-  void *mem=(realloc)(ptr, size);
+  struct memdebug *mem;
+
+  size_t size = sizeof(struct memdebug)+wantedsize;
+
+  mem = (struct memdebug *)((char *)ptr - offsetof(struct memdebug, mem));
+
+  mem=(struct memdebug *)(realloc)(mem, size);
   if(logfile)
     fprintf(logfile, "MEM %s:%d realloc(%p, %d) = %p\n",
-            source, line, ptr, size, mem);
-  return mem;
+            source, line, ptr, wantedsize, mem?mem->mem:NULL);
+
+  if(mem) {
+    mem->size = wantedsize;
+    return mem->mem;
+  }
+
+  return NULL;
 }
 
 void curl_dofree(void *ptr, int line, const char *source)
 {
+  struct memdebug *mem;
+
   if(NULL == ptr) {
     fprintf(stderr, "ILLEGAL free() on NULL at %s:%d\n",
             source, line);
     exit(2);
   }
+  mem = (struct memdebug *)((char *)ptr - offsetof(struct memdebug, mem));
 
-  (free)(ptr);
+  /* destroy  */
+  memset(mem->mem, 0x13, mem->size);
+  
+  /* free for real */
+  (free)(mem);
 
   if(logfile)
     fprintf(logfile, "MEM %s:%d free(%p)\n", source, line, ptr);
@@ -162,13 +202,24 @@ FILE *curl_fopen(const char *file, const char *mode,
 
 int curl_fclose(FILE *file, int line, const char *source)
 {
-  int res=(fclose)(file);
+  int res;
+
+  if(NULL == file) {
+    fprintf(stderr, "ILLEGAL flose() on NULL at %s:%d\n",
+            source, line);
+    exit(2);
+  }
+
+  res=(fclose)(file);
   if(logfile)
     fprintf(logfile, "FILE %s:%d fclose(%p)\n",
             source, line, file);
   return res;
 }
-
+#else
+#ifdef VMS
+int VOID_VAR_MEMDEBUG;	
+#endif
 #endif /* MALLOCDEBUG */
 
 /*
