@@ -1,52 +1,61 @@
 #ifndef __SETUP_H
 #define __SETUP_H
-/*****************************************************************************
+/***************************************************************************
  *                                  _   _ ____  _     
  *  Project                     ___| | | |  _ \| |    
  *                             / __| | | | |_) | |    
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
- *
- * In order to be useful for every potential user, curl and libcurl are
- * dual-licensed under the MPL and the MIT/X-derivate licenses.
+ * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
+ * This software is licensed as described in the file COPYING, which
+ * you should have received as part of this distribution. The terms
+ * are also available at http://curl.haxx.se/docs/copyright.html.
+ * 
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
- * furnished to do so, under the terms of the MPL or the MIT/X-derivate
- * licenses. You may pick one of these licenses.
+ * furnished to do so, under the terms of the COPYING file.
  *
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup.h,v 1.19 2002/01/07 22:47:21 bagder Exp $
- *****************************************************************************/
+ * $Id: setup.h,v 1.25 2002/12/04 11:06:17 bagder Exp $
+ ***************************************************************************/
 
+/* MN 06/07/02 */
+/* #define HTTP_ONLY
+*/
+#ifdef HTTP_ONLY
+#define CURL_DISABLE_FTP
+#define CURL_DISABLE_LDAP
+#define CURL_DISABLE_TELNET
+#define CURL_DISABLE_DICT
+#define CURL_DISABLE_FILE
+#define CURL_DISABLE_GOPHER
+#endif
 
-
-#if !defined(WIN32) && defined(_WIN32)
-/* This _might_ be a good Borland fix. Please report whether this works or
-   not! */
+#if !defined(WIN32) && defined(__WIN32__)
+/* This should be a good Borland fix. Alexander J. Oss told us! */
 #define WIN32
 #endif
 
 #ifdef HAVE_CONFIG_H
 
 #ifdef VMS
-#include "../config-vms.h"
+#include "config-vms.h"
 #else
-#include "../config.h" /* the configure script results */
+#include "config.h" /* the configure script results */
 #endif
 
 #else
 #ifdef WIN32
 /* hand-modified win32 config.h! */
-#include "../config-win32.h"
+#include "config-win32.h"
 #endif
 #ifdef macintosh
 /* hand-modified MacOS config.h! */
-#include "../config-mac.h"
+#include "config-mac.h"
 #endif
 
 #endif
@@ -94,11 +103,12 @@ defined(HAVE_LIBSSL) && defined(HAVE_LIBCRYPTO)
 #ifdef	VMS
 #include "../include/curl/stdcheaders.h"
 #else
-#include "curl/stdcheaders.h"
+#include <curl/stdcheaders.h>
 #endif
+
 #else
 #ifdef _AIX
-#include "curl/stdcheaders.h"
+#include <curl/stdcheaders.h>
 #endif
 #endif
 
