@@ -1,5 +1,5 @@
-#!/usr/bin/perl
-# $Id: runtests.pl,v 1.46 2002/01/30 10:18:47 bagder Exp $
+#!/usr/bin/env perl
+# $Id: runtests.pl,v 1.69 2003/01/09 11:26:57 bagder Exp $
 #
 # Main curl test script, in perl to run on more platforms
 #
@@ -7,6 +7,7 @@
 # These should be the only variables that might be needed to get edited:
 
 use strict;
+#use warnings;
 
 @INC=(@INC, $ENV{'srcdir'}, ".");
 
@@ -20,8 +21,10 @@ my $HTTPSPORT=8433; # this is the HTTPS server port
 my $FTPPORT=8921;  # this is the FTP server port
 my $FTPSPORT=8821;  # this is the FTPS server port
 my $CURL="../src/curl"; # what curl executable to run on the tests
+my $DBGCURL=$CURL; #"../src/.libs/curl";  # alternative for debugging
 my $LOGDIR="log";
 my $TESTDIR="data";
+my $LIBDIR="./libtest";
 my $SERVERIN="$LOGDIR/server.input"; # what curl sent the server
 my $CURLLOG="$LOGDIR/curl.log"; # all command lines run
 my $FTPDCMD="$LOGDIR/ftpserver.cmd"; # copy ftp server instructions here
@@ -48,16 +51,23 @@ my $perl="perl -I$srcdir";
 # this gets set if curl is compiled with memory debugging:
 my $memory_debug=0;
 
+# this gets set if curl is compiled with netrc debugging:
+# It has to be in the global symbol table because of the way 'requires' works
+$main::netrc_debug=0;
+my $netrc_debug = \$main::netrc_debug;
+
 # name of the file that the memory debugging creates:
 my $memdump="memdump";
 
 # the path to the script that analyzes the memory debug output file:
-my $memanalyze="../memanalyze.pl";
+my $memanalyze="./memanalyze.pl";
 
 my $checkstunnel = &checkstunnel;
 
 my $ssl_version; # set if libcurl is built with SSL support
 
+my $skipped=0; # number of tests skipped; reported in main loop
+
 #######################################################################
 # variables the command line options may set
 #
@@ -74,6 +84,10 @@ my $pwd;          # current working directory
 
 chomp($pwd = `pwd`);
 
+# enable memory debugging if curl is compiled with it
+$ENV{'CURL_MEMDEBUG'} = 1;
+$ENV{'HOME'}=$pwd;
+
 #######################################################################
 # Return the pid of the server as found in the given pid file
 #
@@ -140,16 +154,27 @@ sub runhttpserver {
     my $RUNNING;
     my $pid;
 
+    $pid = checkserver ($HTTPPIDFILE);
+
     # verify if our/any server is running on this port
-    my $data=`$CURL --silent -i $HOSTIP:$HOSTPORT/verifiedserver`;
+    my $data=`$CURL --silent -i $HOSTIP:$HOSTPORT/verifiedserver 2>/dev/null`;
 
     if ( $data =~ /WE ROOLZ(: |)(\d*)/ ) {
-        $pid = 0+$2;
+        if($2) {
+            $pid = 0+$2;
+        }
+
+        if(!$pid) {
+            print "Test server already running with unknown pid! Use it...\n";
+            return;
+        }
+
         if($verbose) {
             print "Test server already running with pid $pid, killing it...\n";
         }
     }
     elsif($data ne "") {
+        print "GOT: $data\n";
         print "An alien HTTP server is running on port $HOSTPORT\n",
         "Edit runtests.pl to use another port and rerun the test script\n";
         exit;
@@ -160,7 +185,7 @@ sub runhttpserver {
         }
     }
 
-    if($pid) {
+    if($pid > 0) {
         my $res = kill (9, $pid); # die!
         if(!$res) {
             print "Failed to kill our HTTP test server, do it manually and",
@@ -218,7 +243,7 @@ sub runftpserver {
         my $flag=$debugprotocol?"-v ":"";
 
         # verify that our server is NOT running on this port:
-        my $data=`$CURL --silent -i ftp://$HOSTIP:$FTPPORT/verifiedserver`;
+        my $data=`$CURL --silent -i ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null`;
 
         if ( $data =~ /WE ROOLZ/ ) {
             print "A previous FTP server session is already running and we ",
@@ -238,7 +263,7 @@ sub runftpserver {
         }
 
         # verify that our server is one one running on this port:
-        my $data=`$CURL --silent -i ftp://$HOSTIP:$FTPPORT/verifiedserver`;
+        my $data=`$CURL --silent -i ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null`;
 
         if ( $data !~ /WE ROOLZ/ ) {
             print "Another FTP server is running on port $FTPPORT\n",
@@ -375,14 +400,35 @@ sub displaydata {
         # enabled and we shall verify that no memory leaks exist
         # after each and every test!
         $memory_debug=1;
+
+        # there's only one debug control in the configure script
+        # so hope netrc debugging is enabled and set it up
+        $$netrc_debug = 1;
+        $ENV{'CURL_DEBUG_NETRC'} = 'log/netrc';
     }
     printf("* Memory debugging: %s\n", $memory_debug?"ON":"OFF");
+    printf("* Netrc debugging:  %s\n", $$netrc_debug?"ON":"OFF");
     printf("* HTTPS server:     %s\n", $checkstunnel?"ON":"OFF");
     printf("* FTPS server:      %s\n", $checkstunnel?"ON":"OFF");
     printf("* libcurl SSL:      %s\n", $ssl_version?"ON":"OFF");
     print "***************************************** \n";
 }
 
+#######################################################################
+# substitute the variable stuff into either a joined up file or 
+# a command, in either case passed by reference
+#
+sub subVariables {
+  my ($thing) = @_;
+  $$thing =~ s/%HOSTIP/$HOSTIP/g;
+  $$thing =~ s/%HOSTPORT/$HOSTPORT/g;
+  $$thing =~ s/%HTTPSPORT/$HTTPSPORT/g;
+  $$thing =~ s/%FTPPORT/$FTPPORT/g;
+  $$thing =~ s/%FTPSPORT/$FTPSPORT/g;
+  $$thing =~ s/%SRCDIR/$srcdir/g;
+  $$thing =~ s/%PWD/$pwd/g;
+}
+
 #######################################################################
 # Run a single specified test case
 #
@@ -399,12 +445,41 @@ sub singletest {
         return -1;
     }
 
+    {
+        my %hash = getpartattr("client");
+        my $requires = $hash{'requires'};
+
+        if (defined($requires)) {
+            no strict "refs";
+            my $value=${$requires};
+#            print "This test requires '$requires' with value '$value' \n";
+
+            if (${$requires}) {
+                # this test is OK
+                ;
+            }else {
+                print "$testnum requires $requires, which is not set; skipping\n";
+                $skipped++;
+                return 0;  # look successful
+            }
+        }
+    }
+
+
     # extract the reply data
     my @reply = getpart("reply", "data");
     my @replycheck = getpart("reply", "datacheck");
 
     if (@replycheck) {
         # we use this file instead to check the final output against
+
+        my %hash = getpartattr("reply", "datacheck");
+        if($hash{'nonewline'}) {
+            # Yes, we must cut off the final newline from the final line
+            # of the datacheck
+            chomp($replycheck[$#replycheck]);
+        }
+    
         @reply=@replycheck;
     }
 
@@ -432,7 +507,7 @@ sub singletest {
     # name of the test
     my @testname= getpart("client", "name");
 
-    print "test $testnum...";
+    printf("test %03d...", $testnum);
     if(!$short) {
         my $name = $testname[0];
         $name =~ s/\n//g;
@@ -443,6 +518,14 @@ sub singletest {
         return 0; # look successful
     }
 
+    my @codepieces = getpart("client", "tool");
+
+    my $tool="";
+    if(@codepieces) {
+        $tool = $codepieces[0];
+        chomp $tool;
+    }
+
     # remove previous server output logfile
     unlink($SERVERIN);
 
@@ -456,13 +539,16 @@ sub singletest {
 
     # make some nice replace operations
     $cmd =~ s/\n//g; # no newlines please
-    $cmd =~ s/%HOSTIP/$HOSTIP/g;
-    $cmd =~ s/%HOSTPORT/$HOSTPORT/g;
-    $cmd =~ s/%HTTPSPORT/$HTTPSPORT/g;
-    $cmd =~ s/%FTPPORT/$FTPPORT/g;
-    $cmd =~ s/%FTPSPORT/$FTPSPORT/g;
-    $cmd =~ s/%SRCDIR/$srcdir/g;
-    $cmd =~ s/%PWD/$pwd/g;
+
+    subVariables \$cmd;
+
+#    $cmd =~ s/%HOSTIP/$HOSTIP/g;
+#    $cmd =~ s/%HOSTPORT/$HOSTPORT/g;
+#    $cmd =~ s/%HTTPSPORT/$HTTPSPORT/g;
+#    $cmd =~ s/%FTPPORT/$FTPPORT/g;
+#    $cmd =~ s/%FTPSPORT/$FTPSPORT/g;
+#    $cmd =~ s/%SRCDIR/$srcdir/g;
+#    $cmd =~ s/%PWD/$pwd/g;
 
     #$cmd =~ s/%HOSTNAME/$HOSTNAME/g;
 
@@ -476,11 +562,17 @@ sub singletest {
         my %hash = getpartattr("client", "file");
 
         my $filename=$hash{'name'};
+
         if(!$filename) {
             print "ERROR: section client=>file has no name attribute!\n";
             exit;
         }
-        writearray($filename, \@inputfile);
+        my $fileContent = join('', @inputfile);
+        subVariables \$fileContent;
+#        print "DEBUG: writing file " . $filename . "\n";
+        open OUTFILE, ">$filename";
+        print OUTFILE   $fileContent;
+        close OUTFILE;
     }
 
     my %cmdhash = getpartattr("client", "command");
@@ -492,12 +584,19 @@ sub singletest {
     }
     else {
         if (!@validstdout) {
-            $out="--output $CURLOUT ";
+            $out=" --output $CURLOUT ";
         }
     }
 
-    # run curl, add -v for debug information output
-    my $cmdargs="$out--include -v $cmd";
+    my $cmdargs;
+    if(!$tool) {
+        # run curl, add -v for debug information output
+        $cmdargs ="$out --include -v $cmd";
+    }
+    else {
+        $cmdargs = " $cmd"; # $cmd is the command line for the test file
+        $CURLOUT = $STDOUT; # sends received data to stdout
+    }
 
     my @stdintest = getpart("client", "stdin");
 
@@ -507,11 +606,21 @@ sub singletest {
 
         $cmdargs .= " <$stdinfile";
     }
-    my $CMDLINE="$CURL $cmdargs >$STDOUT 2>$STDERR";
+    my $CMDLINE;
+
+    if(!$tool) {
+        $CMDLINE="$CURL";
+    }
+    else {
+        $CMDLINE="$LIBDIR/$tool";
+        $DBGCURL=$CMDLINE;
+    }
+
+    $CMDLINE .= "$cmdargs >$STDOUT 2>$STDERR";
 
     if($verbose) {
-        print "$CMDLINE\n";
-    }
+        print "$CMDLINE\n"; 
+   }
 
     print CMDLOG "$CMDLINE\n";
 
@@ -522,7 +631,7 @@ sub singletest {
         print GDBCMD "set args $cmdargs\n";
         print GDBCMD "show args\n";
         close(GDBCMD);
-        system("gdb $CURL -x log/gdbcmd");
+        system("gdb $DBGCURL -x log/gdbcmd");
         $res =0; # makes it always continue after a debugged run
     }
     else {
@@ -538,7 +647,11 @@ sub singletest {
 
     if($errorcode || $res) {
         if($errorcode == $res) {
-            if(!$short) {
+            $errorcode =~ s/\n//;
+            if($verbose) {
+                print " received errorcode $errorcode OK";
+            }
+            elsif(!$short) {
                 print " error OK";
             }
         }
@@ -566,8 +679,7 @@ sub singletest {
     }
 
     my %replyattr = getpartattr("reply", "data");
-    if(!$replyattr{'nocheck'} &&
-       @reply) {
+    if(!$replyattr{'nocheck'} && @reply) {
         # verify the received data
         my @out = loadarray($CURLOUT);
         $res = compare(\@out, \@reply);
@@ -602,6 +714,15 @@ sub singletest {
 
         my @protstrip=@protocol;
 
+        # check if there's any attributes on the verify/protocol section
+        my %hash = getpartattr("verify", "protocol");
+
+        if($hash{'nonewline'}) {
+            # Yes, we must cut off the final newline from the final line
+            # of the protocol data
+            chomp($protstrip[$#protstrip]);
+        }
+
         for(@strip) {
             # strip all patterns from both arrays
             @out = striparray( $_, \@out);
@@ -656,7 +777,7 @@ sub singletest {
             print "\n** ALERT! memory debuggin without any output file?\n";
         }
         else {
-            my @memdata=`$memanalyze < $memdump`;
+            my @memdata=`$memanalyze $memdump`;
             my $leak=0;
             for(@memdata) {
                 if($_ ne "") {
@@ -694,63 +815,96 @@ my %run;
 
 sub serverfortest {
     my ($testnum)=@_;
+    my @what;
 
     if($testnum< 100) {
         # 0 - 99 is for HTTP
-        if(!$run{'http'}) {
-            runhttpserver($verbose);
-            $run{'http'}=$HTTPPIDFILE;
-        }
+        push @what, "http";
     }
     elsif($testnum< 200) {
         # 100 - 199 is for FTP
-        if(!$run{'ftp'}) {
-            runftpserver($verbose);
-            $run{'ftp'}=$FTPPIDFILE;
-        }
+        push @what, "ftp";
     }
     elsif($testnum< 300) {
         # 200 - 299 is for FILE, no server!
-        $run{'file'}="moo";
+        push @what, "file";
     }
     elsif($testnum< 400) {
-        # 300 - 399 is for HTTPS, two servers!
-
-        if(!$checkstunnel || !$ssl_version) {
-            # we can't run https tests without stunnel
-            # or if libcurl is SSL-less
-            return 1;
-        }
-
-        if(!$run{'http'}) {
-            runhttpserver($verbose);
-            $run{'http'}=$HTTPPIDFILE;
-        }
-        if(!$run{'https'}) {
-            runhttpsserver($verbose);
-            $run{'https'}=$HTTPSPIDFILE;
-        }
+        # 300 - 399 is for HTTPS
+        push @what, "https";
     }
     elsif($testnum< 500) {
-        # 400 - 499 is for FTPS, also two servers
+        # 400 - 499 is for FTPS
+        push @what, "ftps";
+    }
 
-        if(!$checkstunnel || !$ssl_version) {
-            # we can't run https tests without stunnel
-            # or if libcurl is SSL-less
-            return 1;
+    if(!@what) {
+        # load the test case file definition
+        if(loadtest("${TESTDIR}/test${testnum}")) {
+            if($verbose) {
+                # this is not a test
+                print "$testnum doesn't look like a test case!\n";
+            }
+            return 100;
         }
-        if(!$run{'ftp'}) {
-            runftpserver($verbose);
-            $run{'ftp'}=$FTPPIDFILE;
-        }
-        if(!$run{'ftps'}) {
-            runftpsserver($verbose);
-            $run{'ftps'}=$FTPSPIDFILE;
+        @what = getpart("client", "server");
+
+        if(!$what[0]) {
+            warn "Test case $testnum has no server(s) specified!";
+            return 100;
         }
     }
-    else {
-        print "Bad test number, no server available\n";
-        return 100;
+    for(@what) {
+        my $what = lc($_);
+        $what =~ s/[^a-z]//g;
+        if($what eq "ftp") {
+            if(!$run{'ftp'}) {
+                runftpserver($verbose);
+                $run{'ftp'}=$FTPPIDFILE;
+            }
+        }
+        elsif($what eq "http") {
+            if(!$run{'http'}) {
+                runhttpserver($verbose);
+                $run{'http'}=$HTTPPIDFILE;
+            }
+        }
+        elsif($what eq "ftps") {
+            if(!$checkstunnel || !$ssl_version) {
+                # we can't run https tests without stunnel
+                # or if libcurl is SSL-less
+                return 1;
+            }
+            if(!$run{'ftp'}) {
+                runftpserver($verbose);
+                $run{'ftp'}=$FTPPIDFILE;
+            }
+            if(!$run{'ftps'}) {
+                runftpsserver($verbose);
+                $run{'ftps'}=$FTPSPIDFILE;
+            }
+        }
+        elsif($what eq "file") {
+            # we support it but have no server!
+        }
+        elsif($what eq "https") {
+            if(!$checkstunnel || !$ssl_version) {
+                # we can't run https tests without stunnel
+                # or if libcurl is SSL-less
+                return 1;
+            }
+            if(!$run{'http'}) {
+                runhttpserver($verbose);
+                $run{'http'}=$HTTPPIDFILE;
+            }
+            if(!$run{'https'}) {
+                runhttpsserver($verbose);
+                $run{'https'}=$HTTPSPIDFILE;
+            }
+        }
+        else {
+            warn "we don't support a server for $what";
+        }
     }
     sleep 1; # give a second for the server(s) to startup
     return 0; # ok
@@ -835,37 +989,6 @@ if($testthis[0] ne "") {
     $TESTCASES=join(" ", @testthis);
 }
 
-############################################################################
-#
-# don't let anyone think this works right now
-
-print <<EOM
- ***************************************************************************
-
-    THIS DOES NOT WORK
-
- ***************************************************************************
-
- Things in curl-land have changed, but the test suite has not been fixed
- accordingly and thus, the test suite is currently more or less useless.
-
- *PLEASE* help us fixing this. We have to make our new test server written
- in C work and get used instead of the perl version previously used.
-
- The working version of the test server is found here:
-
- http://curl.haxx.se/dev/sws-0.2.tar.gz
-   
- If you unpack this in the tests/ directory and run the server in there, you
- can actually get test-responses if you do like this:
-
- \$ ./sws 8080 &
- \$ curl localhost:8080/3
-
-EOM
-    ;
-
-
 #######################################################################
 # Output curl version and host info being tested
 #
@@ -916,7 +1039,6 @@ my $failed;
 my $testnum;
 my $ok=0;
 my $total=0;
-my $skipped=0;
 
 foreach $testnum (split(" ", $TESTCASES)) {
 
